@@ -4,57 +4,57 @@ from itertools import permutations
 
 from featuretools import primitives
 from featuretools.feature_base import IdentityFeature
-from featuretools.variable_types import Discrete
 
 logger = logging.getLogger('featuretools')
 
 
 def _get_primitive_options():
     # all possible option keys: function that verifies value type
-    return {'ignore_entities': list_entity_check,
-            'include_entities': list_entity_check,
-            'ignore_variables': dict_to_list_variable_check,
-            'include_variables': dict_to_list_variable_check,
-            'ignore_groupby_entities': list_entity_check,
-            'include_groupby_entities': list_entity_check,
-            'ignore_groupby_variables': dict_to_list_variable_check,
-            'include_groupby_variables': dict_to_list_variable_check}
+    return {'ignore_dataframes': list_dataframe_check,
+            'include_dataframes': list_dataframe_check,
+            'ignore_columns': dict_to_list_column_check,
+            'include_columns': dict_to_list_column_check,
+            'ignore_groupby_dataframes': list_dataframe_check,
+            'include_groupby_dataframes': list_dataframe_check,
+            'ignore_groupby_columns': dict_to_list_column_check,
+            'include_groupby_columns': dict_to_list_column_check}
 
 
-def dict_to_list_variable_check(option, es):
+def dict_to_list_column_check(option, es):
     if not (isinstance(option, dict) and
             all([isinstance(option_val, list) for option_val in option.values()])):
         return False
     else:
-        for entity, variables in option.items():
-            if entity not in es:
-                warnings.warn("Entity '%s' not in entityset" % (entity))
+        for dataframe, columns in option.items():
+            if dataframe not in es:
+                warnings.warn("Dataframe '%s' not in entityset" % (dataframe))
             else:
-                for invalid_var in [variable for variable in variables
-                                    if variable not in es[entity]]:
-                    warnings.warn("Variable '%s' not in entity '%s'" % (invalid_var, entity))
+                for invalid_col in [column for column in columns
+                                    if column not in es[dataframe]]:
+                    warnings.warn("Column '%s' not in dataframe '%s'" % (invalid_col, dataframe))
         return True
 
 
-def list_entity_check(option, es):
+def list_dataframe_check(option, es):
     if not isinstance(option, list):
         return False
     else:
-        for invalid_entity in [entity for entity in option if entity not in es]:
-            warnings.warn("Entity '%s' not in entityset" % (invalid_entity))
+        for invalid_dataframe in [dataframe for dataframe in option if dataframe not in es]:
+            warnings.warn("Dataframe '%s' not in entityset" % (invalid_dataframe))
         return True
 
 
 def generate_all_primitive_options(all_primitives,
                                    primitive_options,
-                                   ignore_entities,
-                                   ignore_variables,
+                                   ignore_dataframes,
+                                   ignore_columns,
                                    es):
-    entityset_dict = {entity.id: [variable.id for variable in entity.variables]
-                      for entity in es.entities}
-    primitive_options = _init_primitive_options(primitive_options, entityset_dict)
-    global_ignore_entities = ignore_entities
-    global_ignore_variables = ignore_variables.copy()
+    dataframe_dict = {dataframe.ww.name: [col for col in dataframe.columns]
+                      for dataframe in es.dataframes}
+
+    primitive_options = _init_primitive_options(primitive_options, dataframe_dict)
+    global_ignore_dataframes = ignore_dataframes
+    global_ignore_columns = ignore_columns.copy()
     # for now, only use primitive names as option keys
     for primitive in all_primitives:
         if primitive in primitive_options and primitive.name in primitive_options:
@@ -65,36 +65,36 @@ def generate_all_primitive_options(all_primitives,
         if primitive in primitive_options or primitive.name in primitive_options:
             options = primitive_options.get(primitive, primitive_options.get(primitive.name))
             # Reconcile global options with individually-specified options
-            included_entities = set().union(*[
-                option.get('include_entities', set()).union(
-                    option.get('include_variables', {}).keys())
+            included_dataframes = set().union(*[
+                option.get('include_dataframes', set()).union(
+                    option.get('include_columns', {}).keys())
                 for option in options])
-            global_ignore_entities = global_ignore_entities.difference(included_entities)
+            global_ignore_dataframes = global_ignore_dataframes.difference(included_dataframes)
             for option in options:
-                # don't globally ignore a variable if it's included for a primitive
-                if 'include_variables' in option:
-                    for entity, include_vars in option['include_variables'].items():
-                        global_ignore_variables[entity] = \
-                            global_ignore_variables[entity].difference(include_vars)
-                option['ignore_entities'] = option['ignore_entities'].union(
-                    ignore_entities.difference(included_entities)
+                # don't globally ignore a column if it's included for a primitive
+                if 'include_columns' in option:
+                    for dataframe, include_cols in option['include_columns'].items():
+                        global_ignore_columns[dataframe] = \
+                            global_ignore_columns[dataframe].difference(include_cols)
+                option['ignore_dataframes'] = option['ignore_dataframes'].union(
+                    ignore_dataframes.difference(included_dataframes)
                 )
-            for entity, ignore_vars in ignore_variables.items():
-                # if already ignoring variables for this entity, add globals
+            for dataframe, ignore_cols in ignore_columns.items():
+                # if already ignoring columns for this dataframe, add globals
                 for option in options:
-                    if entity in option['ignore_variables']:
-                        option['ignore_variables'][entity] = option['ignore_variables'][entity].union(ignore_vars)
-                    # if no ignore_variables and entity is explicitly included, don't ignore the variable
-                    elif entity in included_entities:
+                    if dataframe in option['ignore_columns']:
+                        option['ignore_columns'][dataframe] = option['ignore_columns'][dataframe].union(ignore_cols)
+                    # if no ignore_columns and dataframe is explicitly included, don't ignore the column
+                    elif dataframe in included_dataframes:
                         continue
                     # Otherwise, keep the global option
                     else:
-                        option['ignore_variables'][entity] = ignore_vars
+                        option['ignore_columns'][dataframe] = ignore_cols
         else:
             # no user specified options, just use global defaults
-            primitive_options[primitive] = [{'ignore_entities': ignore_entities,
-                                             'ignore_variables': ignore_variables}]
-    return primitive_options, global_ignore_entities, global_ignore_variables
+            primitive_options[primitive] = [{'ignore_dataframes': ignore_dataframes,
+                                             'ignore_columns': ignore_columns}]
+    return primitive_options, global_ignore_dataframes, global_ignore_columns
 
 
 def _init_primitive_options(primitive_options, es):
@@ -151,59 +151,59 @@ def _init_option_dict(key, option_dict, es):
             initialized_option_dict[option_key] = set(option)
         elif isinstance(option, dict):
             initialized_option_dict[option_key] = {key: set(option[key]) for key in option}
-    # initialize ignore_entities and ignore_variables to empty sets if not present
-    if 'ignore_variables' not in initialized_option_dict:
-        initialized_option_dict['ignore_variables'] = dict()
-    if 'ignore_entities' not in initialized_option_dict:
-        initialized_option_dict['ignore_entities'] = set()
+    # initialize ignore_dataframes and ignore_columns to empty sets if not present
+    if 'ignore_columns' not in initialized_option_dict:
+        initialized_option_dict['ignore_columns'] = dict()
+    if 'ignore_dataframes' not in initialized_option_dict:
+        initialized_option_dict['ignore_dataframes'] = set()
     return initialized_option_dict
 
 
-def variable_filter(f, options, groupby=False):
-    if groupby and not issubclass(f.variable_type, Discrete):
+def column_filter(f, options, groupby=False):
+    if groupby and not f.column_schema.semantic_tags.intersection({'category', 'foreign_key'}):
         return False
-    include_vars = 'include_groupby_variables' if groupby else 'include_variables'
-    ignore_vars = 'ignore_groupby_variables' if groupby else 'ignore_variables'
-    include_entities = 'include_groupby_entities' if groupby else 'include_entities'
-    ignore_entities = 'ignore_groupby_entities' if groupby else 'ignore_entities'
+    include_cols = 'include_groupby_columns' if groupby else 'include_columns'
+    ignore_cols = 'ignore_groupby_columns' if groupby else 'ignore_columns'
+    include_dataframes = 'include_groupby_dataframes' if groupby else 'include_dataframes'
+    ignore_dataframes = 'ignore_groupby_dataframes' if groupby else 'ignore_dataframes'
 
     dependencies = f.get_dependencies(deep=True) + [f]
     for base_f in dependencies:
         if isinstance(base_f, IdentityFeature):
-            if include_vars in options and base_f.entity.id in options[include_vars]:
-                if base_f.get_name() in options[include_vars][base_f.entity.id]:
+            if include_cols in options and base_f.dataframe_name in options[include_cols]:
+                if base_f.get_name() in options[include_cols][base_f.dataframe_name]:
                     continue  # this is a valid feature, go to next
                 else:
                     return False  # this is not an included feature
-            if ignore_vars in options and base_f.entity.id in options[ignore_vars]:
-                if base_f.get_name() in options[ignore_vars][base_f.entity.id]:
+            if ignore_cols in options and base_f.dataframe_name in options[ignore_cols]:
+                if base_f.get_name() in options[ignore_cols][base_f.dataframe_name]:
                     return False  # ignore this feature
-        if include_entities in options:
-            return base_f.entity.id in options[include_entities]
-        elif ignore_entities in options and \
-                base_f.entity.id in options[ignore_entities]:
-            return False  # ignore the entity
+        if include_dataframes in options:
+            return base_f.dataframe_name in options[include_dataframes]
+        elif ignore_dataframes in options and \
+                base_f.dataframe_name in options[ignore_dataframes]:
+            return False  # ignore the dataframe
     return True
 
 
-def ignore_entity_for_primitive(options, entity, groupby=False):
-    # This logic handles whether given options ignore an entity or not
-    def should_ignore_entity(option):
+def ignore_dataframe_for_primitive(options, dataframe, groupby=False):
+    # This logic handles whether given options ignore an dataframe or not
+    def should_ignore_dataframe(option):
         if groupby:
-            if 'include_groupby_variables' not in option or entity.id not in option['include_groupby_variables']:
-                if 'include_groupby_entities' in option and entity.id not in option['include_groupby_entities']:
+            if 'include_groupby_columns' not in option or dataframe.ww.name not in option['include_groupby_columns']:
+                if 'include_groupby_dataframes' in option and dataframe.ww.name not in option['include_groupby_dataframes']:
                     return True
-                elif 'ignore_groupby_entities' in option and entity.id in option['ignore_groupby_entities']:
+                elif 'ignore_groupby_dataframes' in option and dataframe.ww.name in option['ignore_groupby_dataframes']:
                     return True
-        if 'include_variables' in option and entity.id in option['include_variables']:
+        if 'include_columns' in option and dataframe.ww.name in option['include_columns']:
             return False
-        elif 'include_entities' in option:
-            return entity.id not in option['include_entities']
-        elif entity.id in option['ignore_entities']:
+        elif 'include_dataframes' in option:
+            return dataframe.ww.name not in option['include_dataframes']
+        elif dataframe.ww.name in option['ignore_dataframes']:
             return True
         else:
             return False
-    return any([should_ignore_entity(option) for option in options])
+    return any([should_ignore_dataframe(option) for option in options])
 
 
 def filter_groupby_matches_by_options(groupby_matches, options):
@@ -216,13 +216,13 @@ def filter_matches_by_options(matches, options, groupby=False, commutative=False
     # If more than one option, than need to handle each for each input
     if len(options) > 1:
         def is_valid_match(match):
-            if all([variable_filter(m, option, groupby) for m, option in zip(match, options)]):
+            if all([column_filter(m, option, groupby) for m, option in zip(match, options)]):
                 return True
             else:
                 return False
     else:
         def is_valid_match(match):
-            if all([variable_filter(f, options[0], groupby) for f in match]):
+            if all([column_filter(f, options[0], groupby) for f in match]):
                 return True
             else:
                 return False
